// PROJECT:  GRAVOMETER
// PURPOSE:  To (crudely) approximate acceleration due to gravity (-9.82 m/s^2)
// PROCEDURE:Wire up a photoInterrupt (eg. GP1A57HR) triggered on a RISING edge of a free-falling calibrated ruler
//           After a few calculations, the digits are ready to be shifted (HEF4794B) into the segmented display
// AUTHORS:  C. D'Arcy & M. Elia
int data = 2;
int pulse = 4;
int strobe = 5;
int interruptPin = 1;  // aka PIN 3 (ATmega328P Pin 5)
// 7-Segment map: xbafcgde
int digits[] = {
  B01111011,B01001000,B01100111,B01101110,B01011100,B00111110,B00111111,B01101000,B01111111,B01111100}; 
// default digits to present at opening
int units = 0;
int tenths =0;
int hundredths= 0;

int readings = 11;
volatile long splits[11];
volatile int split = 0;
volatile boolean completed = false;
// These are the distances between each of the marks on the ruler M. Elia made
double marks[] = {
  0,2.25,2.45,2.4,2.6,2.5,2.5,2.5,2.5,2.5,2.5};

void setup(){
  pinMode(data,OUTPUT);
  pinMode(pulse,OUTPUT);
  pinMode(strobe,OUTPUT);
  digitalWrite(interruptPin, LOW);
  attachInterrupt(interruptPin, service, FALLING);
  present();
  //  Serial.begin(9600);
}

void service(){
  splits[split++] = micros();
  completed = split==readings;
  //  Serial.println(split);
}

void loop(){
  if (completed){
    completed = false;
    detachInterrupt(interruptPin);
    //    for(units=0; units<10; units++)
    //    for(tenths=0; tenths<10; tenths++)
    //       for( hundredths=0; hundredths<10; hundredths++){
    analyze();
    present();
  } 
}
/*********************************************************************
 * Deduces an approximation of the acceleration from the ruler marks *
 * an interrupt-generated split times.                               *
 ********************************************************************/
void analyze(){
  double total = 0;
  double acceleration[11]={0,0,0,0,0,0,0,0,0,0,0 };
  for(int i=2;i<readings; i++){
    acceleration[i] = (marks[i]/(splits[i]-splits[i-1])-marks[i-1]/(splits[i-1]-splits[i-2]))/(splits[i]-splits[i-1])*1E10;
    total += acceleration[i];
  }
  int averageAcceleration = (int) (total/(readings-2)*100);
  hundredths = averageAcceleration % 10;
  tenths = (averageAcceleration/10) % 10;
  units = averageAcceleration/100;
}

/*************************************************************************
 * Shifts out the three digits for presentation on the segmented display *
 *************************************************************************/
void present(){
  digitalWrite(strobe,LOW);
  shiftOut(data,pulse,MSBFIRST,digits[hundredths]); 
  shiftOut(data,pulse,MSBFIRST,digits[tenths]);  
  shiftOut(data,pulse,MSBFIRST,digits[units]); 
  digitalWrite(strobe,HIGH);
}









